from .model_zoo import get_model
from .arcface_onnx import ArcFaceONNX
from .arcface_trt import ArcFaceTrt
from .retinaface import RetinaFace
from .retinaface_trt import RetinaFaceTrt
from .scrfd import SCRFD
from .landmark import Landmark
from .attribute import Attribute
